#!/usr/bin/env python3

# Unit Tests for apbootstrap.py

import sys
import os
import errno
import tempfile
import unittest
import hashlib

from apbootstrap import Bootstrap

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))


class APBootstrapTests(unittest.TestCase):

    # Create temporary files, test data and populate temp files with test data
    def setUp(self):
        self.inisuffix = ".ini"
        self.csvsuffix = ".csv"

        self.tmpdir_test = self.__create_tmp_dir()
        self.tmp_apbootstrap_ini = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_apbootstrap_ini_comparison = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_linux_bootstrap_ini = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_autopilot_ini = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_autopilot_ini_comparison = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_global_environment_sh = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_global_environment_sh_comparison = self.__create_tmp_file(self.tmpdir_test.name, "", self.inisuffix)
        self.tmp_apt_proxy_file = self.__create_tmp_file(self.tmpdir_test.name, "", "")
        self.tmp_initial_boot_csv = self.__create_tmp_file(self.tmpdir_test.name, "", self.csvsuffix)
        self.tmp_initial_boot_csv_destination = self.__create_tmp_file(self.tmpdir_test.name, "", self.csvsuffix)

        self.apt_proxy_server = "testproxy.proxy.com"
        # Options for sections to reset:
        # initial-boot, autopilot_ini, apt-proxy
        self.sections_to_reset = ["initial-boot","autopilot_ini","apt-proxy"]

        self.data_linux_bootstrap_ini = '''[Config]
Phase=2
Mode=1
ForceClean=false
Verbose=true
WimToolMode=1
WimToolPath=(null)
DiskpartNormalDelayInSec=15
DiskpartOnlineDelayInSec=60
ProvisioningList=azlinux-Dev-CO4$LINUX.Dell14-i20b,azlinux-Dev-CO4$Dell14-i20b,LINUX.Dell14-i20b,Dell14-i20b,Default
CurrentWorkingDirectory=Linux\\
OsImageDirectoryName=(null)
OsImageFileName=(null)
OsImagePathName=D:\OSImage\Azlinux.Ubuntu-16.04.wim
ImageVolumeDriveLetter=D:
DataVolumeDriveLetter=W
EfiSystemVolumeDriveLetter=U
SystemVolumeDriveLetter=V
AutopilotEnvironment=azlinux-Dev-CO4
AutopilotMachineFunction=LINUX
AutopilotHardwareSku=Dell14-i20b
AutopilotCluster=CO4
AutopilotHostname=CO4PHY050080648
AutopilotIPAddress=25.65.157.116
AutopilotIPDNSPrimary=100.101.20.152
AutopilotIPDNSSecondary=(null)
AutopilotAPCAVip=10.8.66.3
AutopilotDeviceName=CO4PHY050080648
AutopilotDeviceImageName=Linux.retail.amd64
AutopilotDistributionServers=100.101.65.222,100.101.65.225,100.101.90.222,100.101.103.222,100.101.78.26,100.101.20.161,100.101.41.90,100.101.103.225,100.101.20.158
AutopilotProvisioningServers=100.101.103.216,100.101.20.152,100.101.90.216,100.101.65.216
AutopilotFilesyncPort=6798'''

        self.data_initial_autopilot_ini = '''[FILES]
DataDir="D:\data"
NetworkDataDir=driveD\Data
#Environment=Autopilot
Environment=<FILL_ENV_HERE>
#Cluster=APTest04
Cluster=<FILL_CLUSTER_HERE>

[MCPClient]
MCPServer=CCPPHY010233424 CCPSCH010233408 CCPSCH010261946
Mode=MCP

[ClusterDM]
DMServer=10.208.91.34 10.208.91.18 10.208.83.56
DMServerNames=CCPPHY010233424 CCPSCH010233408 CCPSCH010261946

[FSSClient]
FSSPORT=6798
FSSSecurePORT=6698

[APCA]
Vip=<FILL_VIP_IP_HERE>

[ApKeyRepository]
Vip=10.29.171.12

'''

        self.data_autopilot_ini_comparison = '''[FILES]
DataDir="D:\data"
NetworkDataDir=driveD\Data
Environment=azlinux-Dev-CO4
Cluster=CO4

[MCPClient]
MCPServer=CCPPHY010233424 CCPSCH010233408 CCPSCH010261946
Mode=MCP

[ClusterDM]
DMServer=10.208.91.34 10.208.91.18 10.208.83.56
DMServerNames=CCPPHY010233424 CCPSCH010233408 CCPSCH010261946

[FSSClient]
FSSPORT=6798
FSSSecurePORT=6698

[APCA]
Vip=10.8.66.3

[ApKeyRepository]
Vip=10.29.171.12

'''

        self.data_global_environment_sh = '''export AP_CLUSTER="CO4"
export AP_ENVIRONMENT="azlinux-Dev-CO4"
export AP_MACHINE_FUNCTION="LINUX"
'''

        self.data_apbootstrap_ini = '''[autopilot_ini]
LastRun=2016-11-01 12:46:04
Success=True
Execute=False

[initial-boot]
LastRun=2016-11-29 16:17:32
Execute=False
Success=True

[apt-proxy]
LastRun=2016-11-29 16:17:32
Execute=False
Success=True'''

        self.assertTrue(self.__populate_file(self.tmp_apbootstrap_ini_comparison.name, self.data_apbootstrap_ini))
        self.assertTrue(self.__populate_file(self.tmp_linux_bootstrap_ini.name, self.data_linux_bootstrap_ini))
        self.assertTrue(self.__populate_file(self.tmp_autopilot_ini.name, self.data_initial_autopilot_ini))
        self.assertTrue(self.__populate_file(self.tmp_autopilot_ini_comparison.name, self.data_autopilot_ini_comparison))
        self.assertTrue(self.__populate_file(self.tmp_global_environment_sh_comparison.name, self.data_global_environment_sh))


    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmp_apbootstrap_ini)
        self.__silent_delete_file(self.tmp_apbootstrap_ini_comparison)
        self.__silent_delete_file(self.tmp_linux_bootstrap_ini)
        self.__silent_delete_file(self.tmp_autopilot_ini)
        self.__silent_delete_file(self.tmp_autopilot_ini_comparison)
        self.__silent_delete_file(self.tmp_global_environment_sh)
        self.__silent_delete_file(self.tmp_global_environment_sh_comparison)
        self.__silent_delete_file(self.tmp_apt_proxy_file)
        self.__silent_delete_file(self.tmp_initial_boot_csv)
        self.__silent_delete_file(self.tmp_initial_boot_csv_destination)
        self.__silent_delete_dir(self.tmpdir_test)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Populate tmp csv file with prepared data
    @staticmethod
    def __populate_file(file: tempfile.NamedTemporaryFile, data: str) -> bool:

        # print("\nTest Configuration Data:")
        # print(data)

        try:
            with open(file, 'w', newline="\r\n") as outfile:
                outfile.write(data)
        except Exception as e:
            print("test_machinelocalwatchdog.__populate_csv_file: Exception: " + str(e))
            return False

        return True

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Compare all file checksums
    def __compare_file_sha256_checksums(self, files_to_compare):
        results = []

        # Loop through all files being compared and place their checksum into the results[] list
        for idx, file in enumerate(files_to_compare):
            try:
                with open(file, "rb") as infile:
                    file_data = infile.read()
                    print("\nFile" + str(idx) + " Data:")
                    print(file_data)
            except Exception as e:
                print("test_machinelocalwatchdog.__compare_file_sha256_checksums: Exception: " + str(e))
                return False

            hash = hashlib.new("sha256")
            hash.update(file_data)
            file_hash = hash.hexdigest()
            print("\nFile" + str(idx) + " Hash: " + str(file_hash))

            results.append(file_hash)

        # Convert to a set which elliminates all duplicate values in a list.  If all values in the list match
        # then the set will contain exactly one element, which is what we're looking for.
        # If more than one element exists that means more than one checksum was found and the files are not the same
        if len(set(results)) != 1:
            return False

        return True

    # Test a complete run of apbootstrap with empty result files so it has to read
    # everything and populate new files
    def test_apbootstrap_full_run(self):
        print("test_apbootstrap_full_run")
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

    # Test to make sure we can reset sections successfully
    def test_reset_sections(self):
        print("test_reset_sections")

        print("\n\nStep1")
        # Generate normal bootstrap.ini data with first run
        # initial-boot
        #     Success : True
        #     Execute : False
        #     LastRun : 2016-11-29 17:27:35
        # autopilot_ini
        #     Success : True
        #     Execute : False
        #     LastRun : 2016-11-29 17:27:35
        # apt-proxy
        #     Success : True
        #     Execute : False
        #     LastRun : 2016-11-29 17:27:35

        # Do first run to initialize all sections or we can't reset them
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

        print("\n\nStep2")
        # Attempt to reset sections
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     self.sections_to_reset,  # this field feeds in sections to reset
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

        print("\n\nStep3")
        # initial-boot
        #     Success : False
        #     LastRun : 2016-11-29 19:20:46
        #     Execute : True
        # autopilot_ini
        #     Success : False
        #     LastRun : 2016-11-29 19:20:46
        #     Execute : True
        # apt-proxy
        #     Success : False
        #     LastRun : 2016-11-29 19:20:46
        #     Execute : True
        #
        # Verify that all sections are reset properly
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        self.assertTrue(self.apbootstrap.check_apbootstrap_ini_exists())
        self.assertTrue(self.apbootstrap.read_apbootstrap_ini())
        self.assertTrue(self.apbootstrap.check_default_sections_options_exist())

        apbootstrap_sections = self.apbootstrap.get_apbootstrap_all_sections()

        # Make sure that all sections to reset are set to Execute: True
        for section in apbootstrap_sections:
            self.assertTrue(self.apbootstrap.check_if_section_should_be_executed(section))

        # Make sure that all sections to reset are set to Success: False
        for section in apbootstrap_sections:
            self.assertFalse(self.apbootstrap.check_if_section_execution_succeeded(section))

        print("\n\nStep4")
        # initial-boot
        #     Success : True
        #     Execute : False
        #     LastRun : 2016-11-29 17:27:35
        # autopilot_ini
        #     Success : True
        #     Execute : False
        #     LastRun : 2016-11-29 17:27:35
        # apt-proxy
        #     Success : True
        #     Execute : False
        #     LastRun : 2016-11-29 17:27:35

        # Do a full run to reset all sections
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

        print("\n\nStep5")
        # Verify that all sections are re-executed properly after reset
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        self.assertTrue(self.apbootstrap.check_apbootstrap_ini_exists())
        self.assertTrue(self.apbootstrap.read_apbootstrap_ini())
        self.assertTrue(self.apbootstrap.check_default_sections_options_exist())

        apbootstrap_sections = self.apbootstrap.get_apbootstrap_all_sections()

        # Make sure that all sections to reset are set to Execute: True
        for section in apbootstrap_sections:
            self.assertFalse(self.apbootstrap.check_if_section_should_be_executed(section))

        # Make sure that all sections to reset are set to Success: False
        for section in apbootstrap_sections:
            self.assertTrue(self.apbootstrap.check_if_section_execution_succeeded(section))

    # Tests to make sure autopilot.ini and ENV VARS result in correctly formatted files with the exact data we put
    # into them and the correct format
    def test_file_comparisons(self):
        # Initialize all files and sections
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

        # Compare idea autopilot.ini data vs autopilot.ini data after apbootstrap runs against
        # the sample autopilot.ini we deploy without variables filled in
        # If apbootstrap ran correctly, they should match exactly
        files_to_compare = [self.tmp_autopilot_ini.name, self.tmp_autopilot_ini_comparison.name]
        self.assertTrue(self.__compare_file_sha256_checksums(files_to_compare))

        # Compare environment variables written vs the expected values
        # If apbootstrap ran correctly, they should match exactly
        files_to_compare = [self.tmp_global_environment_sh.name, self.tmp_global_environment_sh_comparison.name]
        self.assertTrue(self.__compare_file_sha256_checksums(files_to_compare))

    # Tests to make sure that we don't keep executing over and over after initial run
    def test_unecessary_run(self):
        print("test_unecessary_run")

        # Check to make sure that if sections don't exist in apbootstrap.ini they don't attempt to execute
        print("\n\nStep1")
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        self.assertTrue(self.apbootstrap.check_apbootstrap_ini_exists())
        self.assertTrue(self.apbootstrap.read_apbootstrap_ini())
        self.assertFalse(self.apbootstrap.check_default_sections_options_exist())

        apbootstrap_sections = self.apbootstrap.get_apbootstrap_all_sections()

        for section in apbootstrap_sections:
            self.assertFalse(self.apbootstrap.check_if_section_should_be_executed(section))

        # Run a normal run of apbootstrap.main() in order to populate all the files and execute everything
        print("\n\nStep2")
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

        # Check to make sure that if sections have already been run they do not attempt to execute again
        print("\n\nStep3")
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini_comparison.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        self.assertTrue(self.apbootstrap.check_apbootstrap_ini_exists())
        self.assertTrue(self.apbootstrap.read_apbootstrap_ini())
        self.assertTrue(self.apbootstrap.check_default_sections_options_exist())

        apbootstrap_sections = self.apbootstrap.get_apbootstrap_all_sections()

        for section in apbootstrap_sections:
            self.assertFalse(self.apbootstrap.check_if_section_should_be_executed(section))

    def test_sections_to_reset_match(self):
        print("test_sections_to_reset_match")

        # Do first run to initialize all sections
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        # Make sure apbootstrap exited with an exit code of 0
        self.assertRaisesRegexp(SystemExit, '0', self.apbootstrap.main)

        # Run a partial run to get section names defined in apbootstrap
        self.apbootstrap = Bootstrap(self.tmp_apbootstrap_ini.name,
                                     self.tmp_linux_bootstrap_ini.name,
                                     self.tmp_autopilot_ini.name,
                                     self.tmp_global_environment_sh.name,
                                     self.tmp_apt_proxy_file.name,
                                     self.apt_proxy_server,
                                     "",
                                     self.tmp_initial_boot_csv.name,
                                     self.tmp_initial_boot_csv_destination.name,
                                     self.tmpdir_test.name)

        self.assertTrue(self.apbootstrap.check_apbootstrap_ini_exists())
        self.assertTrue(self.apbootstrap.read_apbootstrap_ini())
        self.assertTrue(self.apbootstrap.check_default_sections_options_exist())

        apbootstrap_sections = self.apbootstrap.get_apbootstrap_all_sections()

        # Compare sections in the unittest to those in apbootstrap and make sure they're all in both
        # If the are missing in the unittest, update the unittest!
        self.assertTrue(self.sections_to_reset == apbootstrap_sections)

if __name__ == '__main__':
    unittest.main()

