#!/usr/bin/env python3

"""
Helper functions for APLinux bootstrap secrets.
"""

from http.client import HTTPSConnection
import sys
import os
import stat
import ssl
import json

import keyvaulthelper

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))

from apsdk import apenv

secret_directory = "/var/cache/apsecret"


def get_apsecret_keyvault_connection_data() -> (str, str, str, str):
    """
    Gets the connection data needed to connect to APLinux Prod's KeyVault instance for the purpose of accessing
    bootstrap secrets.
    :return: Returns a tuple containing the TenantID, ClientID, ClientSecret, and VaultURI in that order.
    """
    # retrieve the json config file
    cert, key = apenv.get_machine_function_cert()
    if cert is None or key is None:
        raise ValueError("Unable to find AP machine function certificate")
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
    try:
        conn = HTTPSConnection('aplinuxservices.trafficmanager.net:1443',
                               key_file=key,
                               cert_file=cert,
                               context=ctx,
                               timeout=5)
        conn.putrequest('GET', '/keyvault/aplinux-keyvault.json')
        conn.endheaders()
        response = conn.getresponse()
    except Exception as e:
        raise ValueError("Unable to connect to the APSecret service: %r" % e)

    if response.status != 200:
        raise ValueError("APSecret service returned with an error: %s %s" % (response.status, response.reason))

    try:
        data = response.read()
        conn.close()
    except Exception as e:
        raise ValueError("Error during connection to APSecret service: %r" % e)

    # parse the json config file
    try:
        config = json.loads(data.decode(encoding='UTF-8'))
    except:
        raise ValueError('Connection information retrieved from authority is in a unsupported format.')
    if config is None or len(config) == 0:
        raise ValueError('No connection information available from authority.')
    if [o for o in ('TenantID', 'ClientID', 'ClientSecret', 'VaultURI') if o not in config[0]]:
        raise ValueError('Connection information retrieved from authority is incomplete.')

    return config[0]['TenantID'], config[0]['ClientID'], config[0]['ClientSecret'], config[0]['VaultURI']


def update_secret_cache(name: str) -> bool:
    """
    Updates a locally cached secret.
    :return: Returns true on successful update and false if unable to update secret.
    """
    try:
        tenant_id, client_id, client_secret, vault_uri = get_apsecret_keyvault_connection_data()
    except Exception as e:
        return False

    if len(tenant_id) == 0 or len(client_id) == 0 or len(client_secret) == 0 or len(vault_uri) == 0:
        return False

    config_file_name = os.path.join(secret_directory, name)
    try:
        kv = keyvaulthelper.KeyVaultHelper(tenant_id, client_id, client_secret, vault_uri)
        secret = kv.get_secret(name)

        f = open(config_file_name, 'w+')
        f.write(secret)
        f.close()

        os.chmod(config_file_name, stat.S_IRWXU | stat.S_IRGRP)
    except Exception as e:
        return False

    return True


def get_secret_from_cache(name: str) -> str:
    """
    Gets the value of the specified locally cached secret.
    :return: Returns the secret on success and an empty string if the secret hasn't been cached.
    """
    config_file_name = os.path.join(secret_directory, name)
    try:
        f = open(config_file_name, 'r')
        secret = f.read()
        f.close()
    except Exception as e:
        return ""

    return secret

