#!/usr/bin/env python3

# Unit Tests for statsdhelper.py

import sys
import os
import errno
import tempfile
import unittest
import socket
import json

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))

from apcommon import statsdhelper


class StatsdHelperTests(unittest.TestCase):

    # Create temporary files, test data and populate temp files with test data
    def setUp(self):
        self.hostname = socket.gethostname()

        self.csvsuffix = ".csv"

        self.tmpcsvdir_test = self.__create_tmp_dir()
        self.tmpcsvfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.csvsuffix)

        self.test_data = {"input_csv_file": self.tmpcsvfile_test.name,
                          "Namespace": "testNamespace",
                          "Metric": "testMetric",
                          "Description": "testDescription",
                          "Tenant": "testTenant",
                          "Role": "testRole",
                          "RoleInstance": "testRoleInstance",
                          "MachineName": self.hostname,
                          "OSVersion": "testVersion",
                          "SharedServicesVersion": "testServicesVersion",
                          "TestValue": 5}

        self.csv_data = "MachineName,Ok,OSImageVersion," + self.test_data["OSVersion"] + "\n"
        self.csv_data += "MachineName,Warning,LinuxSharedServicesVersion," + self.test_data["SharedServicesVersion"] + "\n"

        self.assertTrue(self.__populate_csv_file(self.tmpcsvfile_test.name, self.csv_data))

    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmpcsvfile_test)
        self.__silent_delete_dir(self.tmpcsvdir_test)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Populate tmp file with prepared data
    @staticmethod
    def __populate_csv_file(file: tempfile.NamedTemporaryFile, csv_data: str) -> bool:

        print("\nTest Configuration Data:")
        print(csv_data)

        try:
            with open(file, 'w', newline="\r\n") as outfile:
                outfile.write(csv_data)
        except Exception as e:
            print("test_statsdhelper.__populate_csv_file: Exception: " + str(e))
            return False

        return True

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Create a UDP listener on the first available port to listen for statsd data sent by statsdhelper.py
    def __setup_udp_listener(self):
        try:
            self.udp_server = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            self.udp_server.bind(("", 0))
            # self.udp_server.listen(1)
            self.udp_listener_port = self.udp_server.getsockname()[1]
            print("test_statsdhelper.__setup_udp_listener: UDP Server Listening on Port: " + str(self.udp_listener_port))

        except Exception as e:
            print("test_statsdhelper.__setup_udp_listener: Exception: " + str(e))
            return False

        return True

    # Close down the UDP listener
    def __tear_down_udp_listener(self):
        try:
            self.udp_server.close()
            print("test_statsdhelper.__setup_udp_listener: UDP Server Stopped")

        except Exception as e:
            print("test_statsdhelper.__tear_down_udp_listener: Exception: " + str(e))
            return False

        return True

    # Read in UDP data sent during the test
    def __read_udp_data(self):
        buf = 1024
        self.udp_received_data = ""

        data, addr = self.udp_server.recvfrom(buf)
        try:
            while (data):
                # Convert from bytes to string chunks and append to string
                self.udp_received_data += data.decode("utf-8")

                self.udp_server.settimeout(5)

                # Wait x seconds before declaring udp receive data is complete
                # If end of gauge data is detected, set timeout to None
                if data.decode("utf-8").endswith("|g"):
                    print("test_statsdhelper.__read_udp_data: Found End of UDP Data Stream. Stop Reading and Break.")
                    break

                data, addr = self.udp_server.recvfrom(buf)

            self.udp_received_data = self.udp_received_data.strip()

        except socket.timeout:
            print("test_statsdhelper.__read_udp_data: Socket Receive Timed Out. Data Receive Should be Complete.")
        except Exception as e:
            print("test_statsdhelper.__read_udp_data: Exception: " + str(e))
            return False

        return True

    # Create a test JSON to send via statsdhelper.py
    def __create_test_JSON(self):
        metric_dict = {"Namespace": self.test_data["Namespace"],
                       "Metric": self.test_data["Metric"],
                       "Dims": ""}

        dims_dict = {"Description": self.test_data["Description"],
                     "Tenant": self.test_data["Tenant"],
                     "Role": self.test_data["Role"],
                     "RoleInstance": self.test_data["RoleInstance"],
                     "MachineName": self.hostname,
                     "OSVersion": self.test_data["OSVersion"],
                     "SharedServicesVersion": self.test_data["SharedServicesVersion"]}

        metric_dict["Dims"] = dims_dict

        try:
            self.test_json = json.dumps(metric_dict)
            self.test_json += ":" + str(self.test_data["TestValue"]) + "|g"

        except Exception as e:
            print("test_statsdhelper.__create_test_JSON: Exception: " + str(e))
            return False

        return True

    # Make sure the method that gets version information works
    def test_get_version_info(self):
        print("test_get_version_info")
        self.statsdhelper = statsdhelper.StatsdHelper(test_data=self.test_data,
                                                      test_status=True)

        self.assertTrue(self.statsdhelper.get_version_info())

    # Make sure that sending a test gauge works and that the data matches exactly with the data sent
    def test_send_gauge(self):
        print("test_send_gauge")
        self.assertTrue(self.__setup_udp_listener())
        self.statsdhelper = statsdhelper.StatsdHelper(statsd_host="localhost",
                                                      statsd_port=self.udp_listener_port,
                                                      test_data=self.test_data,
                                                      test_status=True)

        self.assertTrue(self.statsdhelper.get_version_info())

        result = self.statsdhelper.send_gauge(self.test_data["Namespace"],
                                              self.test_data["Metric"],
                                              self.test_data["Description"],
                                              self.test_data["TestValue"])

        self.assertTrue(result)

        self.assertTrue(self.__read_udp_data())
        self.assertTrue(self.__tear_down_udp_listener())

        print("\nComparing statsd JSON strings")
        print("\nUDP Received statsd Data:")
        print(self.udp_received_data)

        print("\nUDP Sent statsd gauge JSON:")
        self.assertTrue(self.__create_test_JSON())
        print(self.test_json)

        self.assertTrue(self.udp_received_data == self.test_json)

if __name__ == '__main__':
    unittest.main()




