#!/usr/bin/env python3

# Unit Tests for watchapcertificates.py

import sys
import os
import errno
import tempfile
import unittest

from watchapcertificates import WatchAPCertificates

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))


class WatchAPCertificatesTests(unittest.TestCase):
    # Create temporary files, test data and populate temp files with test data
    def setUp(self):
        self.tmpsuffix = ".tmp"
        self.csvsuffix = ".csv"
        self.crtsuffix = ".crt"

        self.tmpcsvdir_test = self.__create_tmp_dir()
        self.tmpcrtdir_test = self.__create_tmp_dir()

        self.tmpfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.tmpsuffix)
        self.tmpcsvfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.csvsuffix)
        self.tmpmachinecrt_file_test = self.__create_tmp_file(self.tmpcrtdir_test.name, "", self.crtsuffix)
        self.tmpmachine_function_file_test = self.__create_tmp_file(self.tmpcrtdir_test.name, "", self.crtsuffix)

        self.assertTrue(self.__populate_config_file(self.tmpmachinecrt_file_test.name, "machine"))
        self.assertTrue(self.__populate_config_file(self.tmpmachine_function_file_test.name, "machine_function"))

    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmpfile_test)
        self.__silent_delete_file(self.tmpcsvfile_test)
        self.__silent_delete_file(self.tmpmachinecrt_file_test)
        self.__silent_delete_file(self.tmpmachine_function_file_test)

        self.__silent_delete_dir(self.tmpcsvdir_test)
        self.__silent_delete_dir(self.tmpcrtdir_test)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Populate tmp file with prepared data
    @staticmethod
    def __populate_config_file(file: tempfile.NamedTemporaryFile,
                               cert_type: str) -> bool:

        if cert_type == "machine":
            cert_data = '''-----BEGIN CERTIFICATE-----
MIIEmjCCAoKgAwIBAgIQVecbRFU846dOSZJrT407yTANBgkqhkiG9w0BAQsFADAT
MREwDwYDVQQDEwhBUENBLUNPNDAeFw0xNjEwMzAxODIzMjRaFw0xNjEyMjkxODIz
MjRaMBoxGDAWBgNVBAMMD0NPNFBIWTA1MDA4MDY0ODCCASIwDQYJKoZIhvcNAQEB
BQADggEPADCCAQoCggEBAL+6KICIBaqMfdg04mmX2t3GEMc9LO6kvekWBKON60xR
gyr3iJRupIihvs1TmfV7vxZmAecRIgMbApXXhrIHKzbMKoxKBz7dpoC6z3GG9G88
yb9zpkJPJnlap5mSNntpqELIA7gsIXcw0WVIV+KCYiW2vSwsmVFoUXsgl/KzNFKT
hsRMTNZYKgomkmldUrmeZICPMb/3a6OvH02rt8PU/77l0D91MgD0P7/S27CZUWoI
8YV46ST+NJddnKCLBYA/nduuzdbBWjftS6qSxjhga1sCj5bcNqgxdRFXFHNMVQS/
1lPYJVPku9bXPbfZDEyTktBTFinf6IUTQKgCFIt97G0CAwEAAaOB4jCB3zBhBgNV
HQEEWjBYgBCkRtpxGxgiq/mo9EjvAnB7oS8wLTErMCkGA1UEAxMiQVAgUm9vdCBD
ZXJ0aWZpY2F0ZSBBdXRob3JpdHkgMjAxM4ITMwAAACzCa3ijazONLwAAAAAALDAM
BgNVHRMBAf8EAjAAMB4GA1UdEQQXMBWCE0NPNFBIWTA1MDA4MDY0OC5DTzQwHQYD
VR0lBBYwFAYIKwYBBQUHAwEGCCsGAQUFBwMCMA4GA1UdDwEB/wQEAwIEMDAdBgNV
HQ4EFgQUH0IBnY7cOPm4h8wsokS4KdruovIwDQYJKoZIhvcNAQELBQADggIBAHQ5
IzGqcfHEnFKzSe+NN1DgUighiODIE1OkfqYd56qC9RHUYaGm+wfD6lRgKwywngc4
eZ/HN5S92MqXLjm33HTTF5LQuOzttrx8U1eGjgw5yMcOrReoH28MOEbAf+I4NdK3
qTpbOhYR2AFCWePPueF9FMzU+6e5Fdw0bTy7se5cz3srwZPcW9MJDJ8GokkQFDcw
fn0xfDBd8BZF+RmcfFPfdCDVLQsWViebH+mAhfHKYrRup/jIbV3mXswW96qrPisa
/xYwu8E11mC2jZ8FfgWx/dMoSUmstzrP7U6VQJNuuhafuNv0UY2AkOaW3CjFV8ES
iI/WHZnkG3eUTWMLyIdQf9KgXd/j7GSBXZ9QYt+yKYdLfMzj39XFWnAkwAmt5+Qd
DekuIJFr5MjPZ9k6KEhv7KbGc7dwZHp98Q22E/UAinx5/a9LZLiFKA+RwXE5fOfw
+ULEDjwGku8aYU2/X58b6jMea1BrWIK+O0kRBq0QT3MXw9VPXzF6r8rTnozeHtm7
mvi10fcrrdGtM+6mSYlaWCKWbCBjYeEd0sRQPNBhosMxNGpQHxjGSwibfuR2e/rL
kQQ7i0/xzwslkuVfAVEii9J4IbB0IX1EfueRDCdvY3Ix89u3hh8IbYokHm8eFJtY
LaVhwWHZP6AMMoecmcPq90+hg+usdWaygUo+lMVb
-----END CERTIFICATE-----'''

        elif cert_type == "machine_function":
            cert_data = '''-----BEGIN CERTIFICATE-----
MIIEtTCCAp2gAwIBAgIQw13JRXuiuodCPIerZMfOgzANBgkqhkiG9w0BAQsFADAT
MREwDwYDVQQDEwhBUENBLUNPNDAeFw0xNjEwMzAxODIzMjZaFw0xNjEyMjkxODIz
MjZaMBoxGDAWBgNVBAMMD0NPNFBIWTA1MDA4MDY0ODCCASIwDQYJKoZIhvcNAQEB
BQADggEPADCCAQoCggEBAMcAf0OIxL+/coD2Ef/csjjL653ELNcwLXySjKx1vmnZ
5cy1VWbUbI61IZY1fvnRSjfxFbSl5nM+SE4kw4H/c1sTeXEtUGz+MEQk27WYeDUM
uUP9YJ4TMWR7P0AVhoGf6k55DVV8zrjo9FlZiOTCPTSSV506W1EM5Q0JA7d4nuqw
/MdX2952zCO5HVUyXZiemzCtfHdTQiqY2qHHdEVEqHBiQVZ/ub9lOE0MJ6rfDCoU
/jsJqXAnSpEIrZ1ymUfR0WBLG4RUlR/njeVRHgQOR1g65W88IapPbD51e0H1VQl1
TVyyaGU1WYT0YFpg3V/PsfSpTxLuk5lx4cqnMGRaCWECAwEAAaOB/TCB+jBhBgNV
HQEEWjBYgBCkRtpxGxgiq/mo9EjvAnB7oS8wLTErMCkGA1UEAxMiQVAgUm9vdCBD
ZXJ0aWZpY2F0ZSBBdXRob3JpdHkgMjAxM4ITMwAAACzCa3ijazONLwAAAAAALDAM
BgNVHRMBAf8EAjAAMDkGA1UdEQQyMDCCE0NPNFBIWTA1MDA4MDY0OC5DTzSCGUxJ
TlVYLmF6bGludXgtRGV2LUNPNC5DTzQwHQYDVR0lBBYwFAYIKwYBBQUHAwEGCCsG
AQUFBwMCMA4GA1UdDwEB/wQEAwIEMDAdBgNVHQ4EFgQUzmf9JqlJW1lCY72X4i5K
wl2TA0QwDQYJKoZIhvcNAQELBQADggIBAHwFfq8ppOeQX3xl8oSHmAPGUdu5h16b
C7JrNGPBs8aB+nfdyUk2R6BnM3s/OWJwBEJ4OQKFdtHR/6yU9fjwLpELKDE5gvgS
dJ6ECfKOKL4XAeLkf3FwTKEcBcQGA2tAGM8zi/qLGDPzvWpt2gR91tT3mVXYUXHA
7NmtomevsJNdjaiV6r9yaMwaKtbQPmVesTYjgnNU16ZMJMNUBJ0Ww/qE2EsRAjfy
/YhM600GLvuYEZWhtzcLUdvTsK15gyXf7Km3kQl3gQICi7ijKoukWQfMn1JqvKNV
+f0rHLgQEbv7ggvJP85KvCjnVKWqkuyvvnwSlfySMAnTYVQUDVM0KBEZcRXfM0U7
paQWHetPJPUcrz+0kAeEptgb1MOYxCwREiPWltcpoR5/Tx2M8yDwremFjU6lDDPV
5Jm+38aNxqbHfjsxjB+zJvGvB+tIvOPVy7G60OdXgZEHFYMZLeXtHhXyycvWUQ9Y
DYrmp1QkrGRAT7saEhp8S96iCCFIhJGhXnJlqZ1nKfLv+/24uuGvtjoDW753iOcO
WedbSCDcvYS073TL8+2aBs5GNYRzSP58tCge3cL4wiMETutw+QJINd9bnZ/2BOvI
VJshnfxf5y7Lqmg/MZgyJoBUH6LB4mV3jqYXlcJ4s7jl4wx90qiBGvH0e6x/WeAs
tYjwP7NGx8T0
-----END CERTIFICATE-----'''

        else:
            print("test_watchapservicestatus.__populate_config_file: Certificate Type not defined")
            return False

        try:
            with open(file, 'w') as outfile:
                outfile.write(cert_data)
        except Exception as e:
            print("test_watchapservicestatus.__populate_config_file: Exception: " + str(e))
            return False

        return True

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Test that certificate data is read in cleanly
    def test_read_in_certs(self):
        print ("test_read_in_certs")
        self.apcertificates = WatchAPCertificates(self.tmpmachinecrt_file_test.name,
                                             self.tmpmachine_function_file_test.name,
                                             self.tmpcsvdir_test.name,
                                             self.tmpfile_test.name,
                                             self.tmpcsvfile_test.name,
                                             True)
        result = self.apcertificates.read_in_certs()

        self.assertTrue(result)

    # Test that we are able to obtain certificate objects from the raw bytes
    def test_get_cert_objects(self):
        print("test_get_cert_objects")
        self.apcertificates = WatchAPCertificates(self.tmpmachinecrt_file_test.name,
                                                  self.tmpmachine_function_file_test.name,
                                                  self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name,
                                                  self.tmpcsvfile_test.name,
                                                  True)
        self.apcertificates.read_in_certs()

        result = self.apcertificates.get_cert_objects()

        self.assertTrue(result)

    # Test to make sure we can get SAN names from cert objects
    def test_get_all_dns_alt_names(self):
        print("test_get_all_dns_alt_names")
        self.apcertificates = WatchAPCertificates(self.tmpmachinecrt_file_test.name,
                                                  self.tmpmachine_function_file_test.name,
                                                  self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name,
                                                  self.tmpcsvfile_test.name,
                                                  True)
        self.apcertificates.read_in_certs()
        self.apcertificates.get_cert_objects()

        result = self.apcertificates.get_all_dns_alt_names()

        self.assertTrue(result)

    # Test to make sure we can get certificate days remaining
    def test_get_cert_days_remaining(self):
        print("test_get_cert_days_remaining")
        self.apcertificates = WatchAPCertificates(self.tmpmachinecrt_file_test.name,
                                                  self.tmpmachine_function_file_test.name,
                                                  self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name,
                                                  self.tmpcsvfile_test.name,
                                                  True)
        self.apcertificates.read_in_certs()
        self.apcertificates.get_cert_objects()
        self.apcertificates.get_all_dns_alt_names()

        result = self.apcertificates.get_cert_days_remaining()

        self.assertTrue(result)

    # Test that we can correctly build the DM report file
    def test_dm_report(self):
        print("test_dm_report")
        self.apcertificates = WatchAPCertificates(self.tmpmachinecrt_file_test.name,
                                                  self.tmpmachine_function_file_test.name,
                                                  self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name,
                                                  self.tmpcsvfile_test.name,
                                                  True)
        self.apcertificates.read_in_certs()
        self.apcertificates.get_cert_objects()
        self.apcertificates.get_all_dns_alt_names()
        self.apcertificates.get_cert_days_remaining()

        self.assertTrue(self.apcertificates.build_dm_report())
        self.assertTrue(self.apcertificates.verify_dm_report())
        self.assertTrue(self.apcertificates.write_dm_report_to_disk())
        self.assertTrue(self.apcertificates.verify_write_dm_report_to_disk())
        self.assertTrue(self.apcertificates.atomically_rename_dm_report_on_disk())
        self.assertTrue(self.apcertificates.print_dm_report())

if __name__ == '__main__':
    unittest.main()

