#!/usr/bin/env python3

# Unit Tests for watchaphdd.py

import sys
import os
import errno
import tempfile
import unittest

from watchaphdd import WatchAPHDD

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))


class WatchAPHDDTests(unittest.TestCase):
    # Create temporary files, test data and populate temp files with test data
    def setUp(self):
        self.longMessage = True

        self.tmpsuffix = ".tmp"
        self.csvsuffix = ".csv"

        self.tmpcsvdir_test = self.__create_tmp_dir()
        self.tmpfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.tmpsuffix)
        self.tmpcsvfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.csvsuffix)

    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmpfile_test)
        self.__silent_delete_file(self.tmpcsvfile_test)
        self.__silent_delete_dir(self.tmpcsvdir_test)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Test the block device discovery process
    def test_discover_block_devices(self):
        print ("test_discover_block_devices")
        self.aphdd = WatchAPHDD(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                                         self.tmpcsvfile_test.name, True)
        result = self.aphdd.discover_block_devices()
        self.aphdd.print_raw_devices()

        self.assertTrue(result)

    # Test getting the block device type
    def test_get_block_device_type(self):
        print("test_get_block_device_type")
        self.aphdd = WatchAPHDD(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                                         self.tmpcsvfile_test.name, True)
        self.aphdd.discover_block_devices()
        self.aphdd.print_raw_devices()
        result = self.aphdd.get_block_device_type()
        self.aphdd.print_device_results()

        self.assertTrue(result)

    # Test getting the Bus ID of the device
    def test_get_lsscsi_device_bus(self):
        print("test_get_lsscsi_device_bus")
        self.aphdd = WatchAPHDD(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                                         self.tmpcsvfile_test.name, True)
        self.aphdd.discover_block_devices()
        self.aphdd.print_raw_devices()
        self.aphdd.get_block_device_type()
        result = self.aphdd.get_lsscsi_device_bus()
        self.aphdd.print_device_results()

        self.assertTrue(result)

    # Test getting the smartctl Health and Detail information
    def test_get_smartctl_info(self):
        print("test_get_smartctl_info")
        self.aphdd = WatchAPHDD(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                                         self.tmpcsvfile_test.name, True)
        self.aphdd.discover_block_devices()
        self.aphdd.print_raw_devices()
        self.aphdd.get_block_device_type()
        self.aphdd.get_lsscsi_device_bus()
        result = self.aphdd.get_smartctl_info()
        self.aphdd.print_device_results()

        self.assertTrue(result)

    # Test that we can correctly build the DM report file
    def test_dm_report(self):
        print("test_dm_report")
        self.aphdd = WatchAPHDD(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                                         self.tmpcsvfile_test.name, True)
        self.aphdd.discover_block_devices()
        self.aphdd.print_raw_devices()
        self.aphdd.get_block_device_type()
        self.aphdd.get_lsscsi_device_bus()
        self.aphdd.get_smartctl_info()
        self.aphdd.print_device_results()

        self.aphdd.build_dm_report()

        self.assertTrue(self.aphdd.verify_dm_report())

        self.aphdd.write_dm_report_to_disk()

        self.assertTrue(self.aphdd.verify_write_dm_report_to_disk())

        self.aphdd.print_dm_report()

        self.assertTrue(self.aphdd.atomically_rename_dm_report_on_disk())

if __name__ == '__main__':
    unittest.main()

