#!/usr/bin/env python3

# Unit Tests for watchapservicestatus.py

import sys
import os
import errno
import tempfile
import unittest
import json

from watchapservicestatus import WatchAPServiceStatus

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))


class WatchAPServiceStatusTests(unittest.TestCase):
    # Create temporary files, test data and populate temp files with test data
    def setUp(self):
        self.configsuffix = ".conf"
        self.csvsuffix = ".csv"
        self.tmpsuffix = ".tmp"

        self.tmpdir_test = self.__create_tmp_dir()
        self.tmpcsvdir_test = self.__create_tmp_dir()
        self.tmpfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.tmpsuffix)
        self.tmpcsvfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.csvsuffix)
        self.tmpconfigfile_test = self.__create_tmp_file(self.tmpdir_test.name, "", self.configsuffix)
        self.tmpconfigfile2_test = self.__create_tmp_file(self.tmpdir_test.name, "", self.configsuffix)
        self.tmpconfigfile3_test = self.__create_tmp_file(self.tmpdir_test.name, "", self.configsuffix)

        self.assertTrue(self.__populate_config_file(self.tmpconfigfile_test.name, "systemd", "daemon", "testservice"))
        self.assertTrue(self.__populate_config_file(self.tmpconfigfile2_test.name, "systemd", "script", "testservice2"))
        self.assertTrue(self.__populate_config_file(self.tmpconfigfile3_test.name, "docker", "container", "container_name"))

    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmpfile_test)
        self.__silent_delete_file(self.tmpcsvfile_test)
        self.__silent_delete_file(self.tmpconfigfile_test)
        self.__silent_delete_file(self.tmpconfigfile2_test)
        self.__silent_delete_file(self.tmpconfigfile3_test)
        self.__silent_delete_dir(self.tmpcsvdir_test)
        self.__silent_delete_dir(self.tmpdir_test)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Populate tmp file with prepared data
    @staticmethod
    def __populate_config_file(file: tempfile.NamedTemporaryFile,
                               status_source: str,
                               status_type: str,
                               service_name: str) -> bool:

        data = {"config":
                    {"statusSource": status_source,
                     "statusType": status_type,
                     "serviceName": service_name}}

        print("\nTest Configuration Data:")
        print(json.dumps(data, indent=4, sort_keys=True))

        try:
            with open(file, 'w') as outfile:
                json.dump(data, outfile, indent=4)
        except Exception as e:
            print("test_watchapservicestatus.__populate_config_file: Exception: " + str(e))
            return False

        return True

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Test to make sure that JSON config files are properly imported
    def test_config_json_import(self):
        print ("test_config_json_import")
        self.servicestatus = WatchAPServiceStatus(self.tmpdir_test.name, self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name, self.tmpcsvfile_test.name, True)
        result = self.servicestatus.read_conf_files()
        self.servicestatus.print_conf_files()

        self.assertTrue(result)

    # Test to make sure that the JSON format is correct
    def test_config_json_verify(self):
        print("test_config_json_verify")
        self.servicestatus = WatchAPServiceStatus(self.tmpdir_test.name, self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name, self.tmpcsvfile_test.name, True)
        self.servicestatus.read_conf_files()
        self.servicestatus.print_conf_files()
        result = self.servicestatus.verify_conf_files()

        self.assertTrue(result)

    # Test to make sure that the service status is obtained correctly
    def test_get_service_status(self):
        print("test_get_service_status")
        self.servicestatus = WatchAPServiceStatus(self.tmpdir_test.name, self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name, self.tmpcsvfile_test.name, True)
        self.servicestatus.read_conf_files()
        self.servicestatus.verify_conf_files()
        result = self.servicestatus.get_service_status()

        self.servicestatus.print_reported_status()

        self.assertTrue(result)

    # Test that we can correctly build the DM report file
    def test_dm_report(self):
        print("test_dm_report")
        self.servicestatus = WatchAPServiceStatus(self.tmpdir_test.name, self.tmpcsvdir_test.name,
                                                  self.tmpfile_test.name, self.tmpcsvfile_test.name, True)
        self.servicestatus.read_conf_files()
        self.servicestatus.verify_conf_files()
        self.servicestatus.get_service_status()
        self.servicestatus.build_dm_report()

        self.assertTrue(self.servicestatus.verify_dm_report())

        self.servicestatus.write_dm_report_to_disk()

        self.assertTrue(self.servicestatus.verify_write_dm_report_to_disk())

        self.servicestatus.print_dm_report()

        self.assertTrue(self.servicestatus.atomically_rename_dm_report_on_disk())

        result = self.servicestatus.send_statsd_report()

        self.assertTrue(result)

if __name__ == '__main__':
    unittest.main()

