#!/usr/bin/env python3

import sys
import os
import errno
import tempfile
import unittest

from watchapversion import WatchAPVersion

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))


class WatchAPVersionTests(unittest.TestCase):

    def setUp(self):
        # Create temporary files, test data and populate temp files with test data
        self.tmpsuffix = ".tmp"
        self.csvsuffix = ".csv"

        self.tmpcsvdir_test = self.__create_tmp_dir()
        self.tmpfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.tmpsuffix)
        self.tmpcsvfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.csvsuffix)

    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmpfile_test)
        self.__silent_delete_file(self.tmpcsvfile_test)
        self.__silent_delete_dir(self.tmpcsvdir_test)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Test that we can obtain the OS version successfully
    def test_get_os_version(self):
        print ("test_get_os_version")
        self.apversion = WatchAPVersion(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                        self.tmpcsvfile_test.name, "", True)
        result = self.apversion.get_os_version()
        self.apversion.print_version_results()

        self.assertTrue(result)

    # Test that we can obtain the shared services version successfully
    def test_get_shared_services_version(self):
        print("test_get_shared_services_version")
        self.apversion = WatchAPVersion(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                        self.tmpcsvfile_test.name, "", True)
        self.apversion.get_os_version()
        result = self.apversion.get_shared_services_version()
        self.apversion.print_version_results()

        self.assertTrue(result)

    # Test that we can correctly build the DM report file
    def test_dm_report(self):
        print("test_dm_report")
        self.apversion = WatchAPVersion(self.tmpcsvdir_test.name, self.tmpfile_test.name,
                                        self.tmpcsvfile_test.name, "", True)
        self.apversion.get_os_version()
        self.apversion.get_shared_services_version()
        self.apversion.print_version_results()

        self.apversion.build_dm_report()

        self.assertTrue(self.apversion.verify_dm_report())

        self.apversion.write_dm_report_to_disk()

        self.assertTrue(self.apversion.verify_write_dm_report_to_disk())

        self.apversion.print_dm_report()

        self.assertTrue(self.apversion.atomically_rename_dm_report_on_disk())

if __name__ == '__main__':
    unittest.main()

